﻿#region LIOGO - GPL Copyright (c) 2005-2007 Lionel Laské
//
// This file is part of LIOGO.
//
// LIOGO is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// LIOGO is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Liogo; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//================================================================================
#endregion

using System;
using System.ComponentModel;
using System.Globalization;
using MyDLR;


namespace LIOGO.Core
{
    /// <summary>
    /// Liogo Converter. Added to Liogo for MyDLR project.
    /// </summary>
    public class LogoConverter : TypeConverter
    {
        /// <summary>
        /// Does the specified destination could be converted to ILogoValue ?
        /// </summary>
        /// <param name="context">context</param>
        /// <param name="sourceType">source destination</param>
        /// <returns>true if destination can convert to ILogoValue</returns>
        public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
        {
            if (sourceType == typeof(string) || sourceType == typeof(Double)
                || sourceType == typeof(Int32) || sourceType == typeof(bool))
                return true;

            return false;
        }

        /// <summary>
        /// Convert a destination to a ILogoValue.
        /// </summary>
        /// <param name="context">context</param>
        /// <param name="culture">culter</param>
        /// <param name="value">value to convert</param>
        /// <returns>result of conversion</returns>
        public override object ConvertFrom(ITypeDescriptorContext context, CultureInfo culture, object value)
        {
            if (value is string)
                return new LogoWord(value as string);
            else if (value is bool)
                return new LogoWord((bool)value);
            else if (value is Int32)
                return new LogoNumber((int)value);
            else if (value is Double)
                return new LogoNumber((double)value);

            throw new NotSupportedException(value.ToString());
        }

        /// <summary>
        /// Does a ILogoValue could be converted to the specifed destination ?
        /// </summary>
        /// <param name="context">context</param>
        /// <param name="destinationType">destination destination</param>
        /// <returns>true if could be convert</returns>
        public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
        {
            if (destinationType == typeof(string) ||
                destinationType == typeof(bool) ||
                destinationType == typeof(Int32) ||
                destinationType == typeof(double))
                return true;
            else
                return false;
        }

        /// <summary>
        /// Convert to a destination.
        /// </summary>
        /// <param name="context">context</param>
        /// <param name="culture">culture</param>
        /// <param name="value">value to convert</param>
        /// <param name="destinationType">destination destination</param>
        /// <returns>value to the specified destination</returns>
        public override object ConvertTo(ITypeDescriptorContext context, CultureInfo culture, object value, Type destinationType)
        {
            ILogoValue logovalue = value as ILogoValue;
            if (destinationType == typeof(string))
                return logovalue.Print();
            else if (destinationType == typeof(bool))
                return new LogoWord(logovalue).IsTrue();
            else if (destinationType == typeof(Int32))
                return new LogoNumber(logovalue).ToInt();
            else if (destinationType == typeof(Double))
                return new LogoNumber(logovalue).ToDouble();

            throw new NotSupportedException(destinationType.ToString());
        }
    }
}

