﻿using System;
using MyDLR;

namespace LIOGO.Core
{
    /// <summary>
    /// DLR Provider for Liogo.
    /// </summary>
    public class LogoDLRProvider : IDLRProvider
    {
        /// <summary>
        /// Test is the language contain a definition for a class.
        /// </summary>
        /// <param name="classname">class name</param>
        /// <returns>true if language has this class</returns>
        public bool HasClass(string classname)
        {
            string lowername = classname.ToLower();
            if (lowername.Equals("word"))
                return true;
            else if (lowername.Equals("number"))
                return true;

            return false;
        }

        /// <summary>
        /// Create an instance of the class using parameters values.
        /// </summary>
        /// <param name="classname">class name</param>
        /// <param name="args">arguments for constructor</param>
        /// <returns>the newly created object</returns>
        public object CreateInstance(string classname, params object[] args)
        {
            // Is it a Liogo class ?
            string lowername = classname.ToLower();
            if (!HasClass(classname))
                throw new NotSupportedException(classname);

            // Liogo constructors need one param
            if (args.Length < 1)
                throw new NotSupportedException(classname);
            object param = args[0];
            Type type = param.GetType();

            // Create using base destination
            if (lowername == "word")
            {
                if (type == typeof(string))
                    return new LogoWord(param as string);
                else if (type == typeof(bool))
                    return new LogoWord((bool)param);
                else
                    return new LogoWord(param.ConvertTo(typeof(ILogoValue)) as ILogoValue);
            }
            else if (lowername == "number")
            {
                if (type == typeof(Int32))
                    return new LogoNumber((Int32)param);
                else if (type == typeof(double))
                    return new LogoNumber((double)param);
                else
                    return new LogoNumber(param.ConvertTo(typeof(ILogoValue)) as ILogoValue);
            }

            throw new NotSupportedException(classname);
        }
    }
}
