﻿using System;

namespace MyDLR
{
    /// <summary>
    /// Interface for a dynamically callable object.
    /// </summary>
    public interface ICallable
    {
        /// <summary>
        /// Call a method on the object.
        /// </summary>
        /// <param name="name">method name</param>
        /// <param name="args">parameters</param>
        /// <returns>return value for the method</returns>
        object CallMethod(string name, params object[] args);

        /// <summary>
        /// Get a property value on the object.
        /// </summary>
        /// <param name="name">property name</param>
        /// <returns>property value</returns>
        object GetProperty(string name);

        /// <summary>
        /// Set value for a property on the object.
        /// </summary>
        /// <param name="name">property name</param>
        /// <param name="value">new property value</param>
        void SetProperty(string name, object value);
    }
}
