﻿using MyDLR;
using LIOGO.Core;
using MyJScript;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;

namespace MyDLRUnitTest
{
    /// <summary>
    ///This is a test class for DynamicObjectExtensionTest and is intended
    ///to contain all DynamicObjectExtensionTest Unit Tests
    ///</summary>
    [TestClass()]
    public class DynamicObjectExtensionTest
    {
        /// <summary>
        /// Private fields
        /// </summary>
        object lw;
        object ln;
        object lw2;
        object d;
        object s;
        object mjs;
        object mjn;
        object mjc;

        private TestContext testContextInstance;

        public TestContext TestContext
        {
            get { return testContextInstance; }
            set { testContextInstance = value; }
        }

        /// <summary>
        /// Miaow method for class Cat.
        /// </summary>
        /// <param name="this">this</param>
        /// <param name="args">args</param>
        /// <returns>nothing</returns>
        static MJSObject CatMiaow(MJSObject @this, params MJSObject[] args)
        {
            Console.WriteLine((@this.Get("name").ValueOf() as string) + " said Miaow");
            return null;
        }

        /// <summary>
        /// Constructor for MyJScript class Cat
        /// </summary>
        /// <param name="this">this</param>
        /// <param name="args">args</param>
        /// <returns>object</returns>
        static MJSObject CatConstructor(MJSObject @this, params MJSObject[] args)
        {
            @this.Set("name", args[0]);
            @this.Set("miaow", new MJSFunction(CatMiaow));
            return null;
        }

        [ClassInitialize]
        static public void MyTestClassInitialize(TestContext t)
        {
            // Add providers for each language
            Providers.Add(new MJSDLRProvider());
            Providers.Add(new LogoDLRProvider());

            // Add Cat class to MyJScript
            MJSContext.GetContext().Set("Cat", new MJSFunction(CatConstructor));
        }

        /// <summary>
        /// Use TestInitialize to run code before running each test
        /// </summary>
        [TestInitialize()]
        public void MyTestInitialize()
        {
            mjs = "String".CreateInstance("DotNetGuru");
            mjn = "Number".CreateInstance(21);
            mjc = "Cat".CreateInstance("Felix");
            lw = "Word".CreateInstance("Hello World !");
            ln = "NUMBER".CreateInstance(100);
            lw2 = "WORD".CreateInstance(ln);
            d = "System.DateTime".CreateInstance();
            s = "System.String".CreateInstance(new Char[3] { 'A', 'B', 'C' });
        }
        
        /// <summary>
        /// Use TestCleanup to run code after each test has run
        /// </summary>
        [TestCleanup()]
        public void MyTestCleanup()
        {
        }
        

        /// <summary>
        ///A test for CallMethod
        ///</summary>
        [TestMethod()]
        public void CallMethodTest()
        {
            object res;

            // Without parameter
            res = lw.CallMethod("ButFIRST");
            Assert.IsNotNull(res);
            Assert.AreEqual(typeof(LogoWord), res.GetType());
            Assert.AreEqual("\"|ello World !|", res.ToString());

            res = d.CallMethod("TOlongDateSTRINg");
            Assert.IsNotNull(res);

            res = s.CallMethod("tolower");
            Assert.IsNotNull(res);
            Assert.AreEqual(typeof(string), res.GetType());
            Assert.AreEqual("abc", res.ToString());

            res = mjs.CallMethod("bold");
            Assert.IsNotNull(res);
            Assert.AreEqual(typeof(MJSString), res.GetType());
            Assert.AreEqual("\"<bold>DotNetGuru</bold>\"", res.ToString());

            // Too much parameter
            try
            {
                res = lw.CallMethod("ButFIRST", 2);
                Assert.Fail();
            }
            catch (NotSupportedException)
            {
            }
            try
            {
                res = s.CallMethod("tolower", 13);
                Assert.Fail();
            }
            catch (NotSupportedException)
            {
            }

            res = mjs.CallMethod("blink", "foo");
            Assert.IsNotNull(res);
            Assert.AreEqual(typeof(MJSString), res.GetType());
            Assert.AreEqual("\"<blink>DotNetGuru</blink>\"", res.ToString());

            // Right parameter number and destination
            res = lw.CallMethod("item", new LogoNumber(2));
            Assert.IsNotNull(res);
            Assert.AreEqual(typeof(LogoWord), res.GetType());
            Assert.AreEqual("\"e", res.ToString());

            res = s.CallMethod("indexof", "B");
            Assert.IsNotNull(res);
            Assert.AreEqual(typeof(Int32), res.GetType());
            Assert.AreEqual("1", res.ToString());

            res = mjs.CallMethod("substr", 6);
            Assert.IsNotNull(res);
            Assert.AreEqual(typeof(MJSString), res.GetType());
            Assert.AreEqual("\"Guru\"", res.ToString());

            res = mjs.CallMethod("substr", "3", new LogoNumber(3));
            Assert.IsNotNull(res);
            Assert.AreEqual(typeof(MJSString), res.GetType());
            Assert.AreEqual("\"Net\"", res.ToString());

            // Parameter need to be converted
            res = lw.CallMethod("item", 2);
            Assert.IsNotNull(res);
            Assert.AreEqual(typeof(LogoWord), res.GetType());
            Assert.AreEqual("\"e", res.ToString());

            res = s.CallMethod("indexof", new LogoWord("B"));
            Assert.IsNotNull(res);
            Assert.AreEqual(typeof(Int32), res.GetType());
            Assert.AreEqual("1", res.ToString());

            // Wrong method
            try
            {
                res = lw.CallMethod("xxx");
                Assert.Fail();
            }
            catch (NotSupportedException)
            {
            }

            try
            {
                res = mjs.CallMethod("xxx");
                Assert.Fail();
            }
            catch (NotSupportedException)
            {
            }

            try
            {
                res = mjc.CallMethod("name");
                Assert.Fail();
            }
            catch (NotSupportedException)
            {
            }

            try
            {
                res = d.CallMethod("xxx");
                Assert.Fail();
            }
            catch (NotSupportedException)
            {
            }
        }

        /// <summary>
        ///A test for GetProperty
        /// </summary>
        [TestMethod()]
        public void GetPropertyTest()
        {
            object res;

            res = lw.GetProperty("cOunt");
            Assert.IsNotNull(res);
            Assert.AreEqual(typeof(LogoNumber), res.GetType());
            Assert.AreEqual("13", res.ToString());

            res = mjs.GetProperty("length");
            Assert.IsNotNull(res);
            Assert.AreEqual(typeof(MJSNumber), res.GetType());
            Assert.AreEqual("10", res.ToString());

            res = mjc.GetProperty("name");
            Assert.IsNotNull(res);
            Assert.AreEqual(typeof(MJSString), res.GetType());
            Assert.AreEqual("\"Felix\"", res.ToString());
        }

        /// <summary>
        ///A test for SetProperty
        /// </summary>
        [TestMethod()]
        public void SetPropertyTest()
        {
            object res;

            mjc.SetProperty("name", "blob");
            res = mjc.GetProperty("name");
            Assert.IsNotNull(res);
            Assert.AreEqual(typeof(MJSString), res.GetType());
            Assert.AreEqual("\"blob\"", res.ToString());

            mjc.SetProperty("name", new MJSString("foo"));
            res = mjc.GetProperty("name");
            Assert.IsNotNull(res);
            Assert.AreEqual(typeof(MJSString), res.GetType());
            Assert.AreEqual("\"foo\"", res.ToString());

            mjc.SetProperty("name", "Felix");
            res = mjc.GetProperty("name");
            Assert.IsNotNull(res);
            Assert.AreEqual(typeof(MJSString), res.GetType());
            Assert.AreEqual("\"Felix\"", res.ToString());
        }

        /// <summary>
        ///A test for ConvertTo
        ///</summary>
        [TestMethod()]
        public void ConvertToTest()
        {
            object res;

            res = lw.ConvertTo(Type.GetType("System.String"));
            Assert.IsNotNull(res);
            Assert.AreEqual(typeof(string), res.GetType());
            Assert.AreEqual("Hello World !", res);

            res = ln.ConvertTo(Type.GetType("System.Int32"));
            Assert.IsNotNull(res);
            Assert.AreEqual(typeof(Int32), res.GetType());
            Assert.AreEqual(100, res);

            res = mjs.ConvertTo(typeof(string));
            Assert.IsNotNull(res);
            Assert.AreEqual(typeof(string), res.GetType());
            Assert.AreEqual("DotNetGuru", res);

            res = mjn.ConvertTo(typeof(Int32));
            Assert.IsNotNull(res);
            Assert.AreEqual(typeof(Int32), res.GetType());
            Assert.AreEqual(21, (Int32)res);

            res = ((object)200).ConvertTo(Type.GetType("System.String"));
            Assert.IsNotNull(res);
            Assert.AreEqual(typeof(string), res.GetType());
            Assert.AreEqual("200", res);

            res = ((object)"A string").ConvertTo(typeof(LogoWord));
            Assert.IsNotNull(res);
            Assert.AreEqual(typeof(LogoWord), res.GetType());
            Assert.AreEqual("\"|A string|", res.ToString());

            res = ((object)"A string").ConvertTo(typeof(MJSObject));
            Assert.IsNotNull(res);
            Assert.AreEqual(typeof(MJSString), res.GetType());
            Assert.AreEqual("\"A string\"", res.ToString());

            res = ((object)"A string").ConvertTo(typeof(MJSString));
            Assert.IsNotNull(res);
            Assert.AreEqual(typeof(MJSString), res.GetType());
            Assert.AreEqual("\"A string\"", res.ToString());

            res = ((object)"Nothing to do").ConvertTo(Type.GetType("System.String"));
            Assert.IsNotNull(res);
            Assert.AreEqual(typeof(string), res.GetType());
            Assert.AreEqual("Nothing to do", res);

            res = ((object)45).ConvertTo(Type.GetType("System.ValueType"));
            Assert.IsNotNull(res);
            Assert.AreEqual(typeof(Int32), res.GetType());
            Assert.AreEqual(45, res);

            res = ((object)true).ConvertTo(typeof(LogoWord));
            Assert.IsNotNull(res);
            Assert.AreEqual(typeof(LogoWord), res.GetType());
            Assert.AreEqual("\"true", res.ToString());

            res = ((object)301).ConvertTo(typeof(LogoWord));
            Assert.IsNotNull(res);
            Assert.AreEqual(typeof(LogoNumber), res.GetType());
            Assert.AreEqual("301", res.ToString());

            res = ((object)407).ConvertTo(typeof(MJSObject));
            Assert.IsNotNull(res);
            Assert.AreEqual(typeof(MJSNumber), res.GetType());
            Assert.AreEqual("407", res.ToString());

            res = ((object)4.3).ConvertTo(typeof(LogoWord));
            Assert.IsNotNull(res);
            Assert.AreEqual(typeof(LogoNumber), res.GetType());
            Assert.AreEqual("4.3", res.ToString());

            try
            {
                res = lw.ConvertTo(Type.GetType("System.DateTime"));
                Assert.Fail();
            }
            catch (NotSupportedException)
            {
            }

            try
            {
                res = DateTime.Now.ConvertTo(typeof(LogoWord));
            }
            catch (NotSupportedException)
            {
            }

            try
            {
                res = DateTime.Now.ConvertTo(typeof(MJSObject));
            }
            catch (NotSupportedException)
            {
            }
        }

        /// <summary>
        ///A test for CreateInstance
        ///</summary>
        [TestMethod()]
        public void CreateInstanceTest()
        {
            Assert.IsNotNull(lw);
            Assert.AreEqual(typeof(LogoWord), lw.GetType());
            Assert.AreEqual("\"|Hello World !|", lw.ToString());

            Assert.IsNotNull(ln);            
            Assert.AreEqual(typeof(LogoNumber), ln.GetType());
            Assert.AreEqual("100", ln.ToString());

            Assert.IsNotNull(lw2);
            Assert.AreEqual(typeof(LogoWord), lw2.GetType());
            Assert.AreEqual("\"100", lw2.ToString());

            Assert.IsNotNull(mjs);
            Assert.AreEqual(typeof(MJSString), mjs.GetType());
            Assert.AreEqual("\"DotNetGuru\"", mjs.ToString());

            Assert.IsNotNull(mjn);
            Assert.AreEqual(typeof(MJSNumber), mjn.GetType());
            Assert.AreEqual("21", mjn.ToString());

            Assert.IsNotNull(mjc);
            Assert.AreEqual(typeof(MJSObject), mjc.GetType());
            Assert.AreEqual("{\"name\":\"Felix\", \"miaow\":function() {}}", mjc.ToString());

            Assert.IsNotNull(d);
            Assert.AreEqual(typeof(DateTime), d.GetType());

            Assert.IsNotNull(s);
            Assert.AreEqual(typeof(String), s.GetType());
            Assert.AreEqual("ABC", s.ToString());

            object b = "word".CreateInstance(true);
            Assert.IsNotNull(b);
            Assert.AreEqual(typeof(LogoWord), b.GetType());
            Assert.AreEqual("\"true", b.ToString());

            object it1 = "Cat".CreateInstance(lw);
            Assert.IsNotNull(it1);
            Assert.AreEqual(typeof(MJSObject), it1.GetType());
            Assert.AreEqual("{\"name\":\"Hello World !\", \"miaow\":function() {}}", it1.ToString());

            object it2 = "Word".CreateInstance(mjs);
            Assert.IsNotNull(it2);
            Assert.AreEqual(typeof(LogoWord), it2.GetType());
            Assert.AreEqual("\"DotNetGuru", it2.ToString());

            try
            {
                "xxx".CreateInstance();
                Assert.Fail();
            }
            catch (NotSupportedException)
            {
            }
        }
    }
}
