﻿#region LIOGO - GPL Copyright (c) 2005-2007 Lionel Laské
//
// This file is part of LIOGO.
//
// LIOGO is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// LIOGO is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Liogo; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//================================================================================
#endregion

using System;
#if !TEST_VSNET
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
#endif
using LIOGO.Core;

namespace LIOGO.Test
{
    /// <summary>
    /// Test class for LogoNumber.
    /// </summary>
    [TestClass()]
    [TestFixture]
    public class LogoNumberTest
    {
        /// <summary>
        /// Constructor
        /// </summary>
        public LogoNumberTest()
        {
        }

        /// <summary>
        /// Init test
        /// </summary>
        [TestInitialize()]
        [SetUp]
        public void Init()
        {
        }

        /// <summary>
        /// Test on constructors
        /// </summary>
        [TestMethod()]
        [Test]
        public void Constructor()
        {
            // Integer
            LogoNumber n1 = new LogoNumber(301269);
            Assert.IsNotNull(n1);
            Assert.AreEqual(n1.Show(), "301269");
            Assert.AreEqual(n1.Print(), "301269");
            Assert.IsFalse(n1.IsEmpty());
            Assert.IsFalse(n1.IsWord());
            Assert.IsTrue(n1.IsNumber());
            Assert.IsFalse(n1.IsList());
            Assert.IsFalse(n1.IsArray());

            // Double
            LogoNumber n2 = new LogoNumber(3.14159);
            Assert.IsNotNull(n2);
            Assert.AreEqual(n2.Show(), "3.14159");
            Assert.AreEqual(n2.Print(), 3.14159.ToString());
            Assert.IsFalse(n2.IsEmpty());
            Assert.IsFalse(n2.IsWord());
            Assert.IsTrue(n2.IsNumber());
            Assert.IsFalse(n2.IsList());
            Assert.IsFalse(n2.IsArray());

            // Logo Value
            LogoNumber n3 = new LogoNumber(new LogoWord("301269"));
            Assert.IsNotNull(n3);
            Assert.IsTrue(n3.IsEqual(n1));
            Assert.AreEqual(n3.Show(), "301269");
            Assert.AreEqual(n3.Print(), "301269");
            Assert.IsFalse(n3.IsEmpty());
            Assert.IsFalse(n3.IsWord());
            Assert.IsTrue(n3.IsNumber());
            Assert.IsFalse(n3.IsList());
            Assert.IsFalse(n3.IsArray());

            LogoNumber n5 = new LogoNumber(n2);
            Assert.IsNotNull(n5);
            Assert.IsTrue(n5.IsEqual(n2));
            Assert.AreEqual(n5.Show(), n2.Show());
            Assert.AreEqual(n5.Print(), n2.Print());
            Assert.IsFalse(n5.IsEmpty());
            Assert.IsFalse(n5.IsWord());
            Assert.IsTrue(n5.IsNumber());
            Assert.IsFalse(n5.IsList());
            Assert.IsFalse(n5.IsArray());
            try
            {
                LogoNumber n6 = new LogoNumber(new LogoWord("1.79769313486232e309"));
                Assert.IsTrue(false);
            }
            catch (Exception)
            {
                Assert.IsTrue(true);
            }
        }

        /// <summary>
        /// Test on CreateFromString (string)
        ///</summary>
        [TestMethod()]
        [Test]
        public void CreateFromString()
        {
            Assert.IsNull(LogoNumber.CreateFromString(""));
            Assert.IsNull(LogoNumber.CreateFromString("ABC"));
            Assert.IsNull(LogoNumber.CreateFromString("124E"));
            Assert.IsNull(LogoNumber.CreateFromString("-"));
            Assert.IsNull(LogoNumber.CreateFromString("10F"));
            Assert.IsNull(LogoNumber.CreateFromString("2,456,322"));
            Assert.AreEqual("12", LogoNumber.CreateFromString("12").Print());
            Assert.AreEqual("-1", LogoNumber.CreateFromString("-1").Print());
            Assert.AreEqual((3.1456).ToString(), LogoNumber.CreateFromString("3.1456").Print());
            Assert.AreEqual("-4E+45", LogoNumber.CreateFromString("-4E45").Print());
            Assert.AreEqual("0", LogoNumber.CreateFromString("0").Print());
            Assert.AreEqual("157239", LogoNumber.CreateFromString("157239").Print());
        }

        /// <summary>
        /// Test on CreateFromWord (LogoWord)
        ///</summary>
        [TestMethod()]
        [Test]
        public void CreateFromWord()
        {
            Assert.IsNull(LogoNumber.CreateFromWord(new LogoWord("")));
            Assert.IsNull(LogoNumber.CreateFromWord(new LogoWord("ABC")));
            Assert.IsNull(LogoNumber.CreateFromWord(new LogoWord("124E")));
            Assert.IsNull(LogoNumber.CreateFromWord(new LogoWord("-")));
            Assert.IsNull(LogoNumber.CreateFromWord(new LogoWord("10F")));
            Assert.IsNull(LogoNumber.CreateFromWord(new LogoWord("2,456,322")));
            Assert.AreEqual("12", LogoNumber.CreateFromWord(new LogoWord("12")).Print());
            Assert.AreEqual("-1", LogoNumber.CreateFromWord(new LogoWord("-1")).Print());
            Assert.AreEqual((3.1456).ToString(), LogoNumber.CreateFromWord(new LogoWord("3.1456")).Print());
            Assert.AreEqual("-4E+45", LogoNumber.CreateFromWord(new LogoWord("-4E45")).Print());
            Assert.AreEqual("0", LogoNumber.CreateFromWord(new LogoWord("0")).Print());
            Assert.AreEqual("157239", LogoNumber.CreateFromWord(new LogoWord("157239")).Print());
        }

        /// <summary>
        /// Test on Item operation
        /// </summary>
        [TestMethod()]
        [Test]
        public void Item()
        {
            // Single value
            LogoNumber n1 = new LogoNumber(1234.5678);
            try
            {
                n1.Item(new LogoNumber(-1));
                Assert.IsTrue(false);
            }
            catch (Exception)
            {
                Assert.IsTrue(true);
            }
            try
            {
                n1.Item(new LogoNumber(0));
                Assert.IsTrue(false);
            }
            catch (Exception)
            {
                Assert.IsTrue(true);
            }
            Assert.IsTrue(n1.Item(new LogoNumber(1)).IsEqual(new LogoWord("1")));
            Assert.IsTrue(n1.Item(new LogoNumber(1.5F)).IsEqual(new LogoWord("1")));
            Assert.IsTrue(n1.Item(new LogoNumber(5)).IsEqual(new LogoWord(".")));
            try
            {
                n1.Item(new LogoNumber(10));
                Assert.IsTrue(false);
            }
            catch (Exception)
            {
                Assert.IsTrue(true);
            }
            try
            {
                n1.Item(new LogoNumber(1234.5678));
                Assert.IsTrue(false);
            }
            catch (Exception)
            {
                Assert.IsTrue(true);
            }
            Assert.IsTrue(n1.Item(new LogoWord("1")).IsEqual(new LogoWord("1")));
            Assert.IsTrue(n1.Item(new LogoWord("1.5")).IsEqual(new LogoWord("1")));
        }

        /// <summary>
        /// Test on First operation
        /// </summary>
        [TestMethod()]
        [Test]
        public void First()
        {
            // Single value
            LogoNumber n1 = new LogoNumber(1234.5678);
            Assert.IsTrue(n1.First().IsEqual(new LogoNumber(1)));
            Assert.IsTrue(n1.First().Print().Equals("1"));
            Assert.IsTrue(n1.First().First().First().Print().Equals("1"));
        }

        /// <summary>
        /// Test on Last operation
        /// </summary>
        [TestMethod()]
        [Test]
        public void Last()
        {
            // Single value
            LogoNumber n1 = new LogoNumber(1234.5678);
            Assert.IsTrue(n1.Last().IsEqual(new LogoNumber(8)));
            Assert.IsTrue(n1.Last().Print().Equals("8"));
            Assert.IsTrue(n1.Last().Last().Last().Print().Equals("8"));
        }

        /// <summary>
        /// Test on ButFirst operation
        /// </summary>
        [TestMethod()]
        [Test]
        public void ButFirst()
        {
            // Single value
            LogoNumber n1 = new LogoNumber(3.14);
            Assert.IsTrue(n1.ButFirst().IsEqual(new LogoNumber(0.14)));
            Assert.IsTrue(n1.ButFirst().Print().Equals(".14"));
            Assert.IsTrue(n1.ButFirst().ButFirst().ButFirst().Print().Equals("4"));
            Assert.IsTrue(n1.ButFirst().ButFirst().ButFirst().ButFirst().IsEmpty());
        }

        /// <summary>
        /// Test on ButLast operation
        /// </summary>
        [TestMethod()]
        [Test]
        public void ButLast()
        {
            // Single value
            LogoNumber n1 = new LogoNumber(3.14);
            Assert.IsTrue(n1.ButLast().IsEqual(new LogoNumber(3.1)));
            Assert.IsTrue(n1.ButLast().Print().Equals("3.1"));
            Assert.IsTrue(n1.ButLast().ButLast().ButLast().Print().Equals("3"));
            Assert.IsTrue(n1.ButLast().ButLast().ButLast().ButLast().IsEmpty());
        }

        /// <summary>
        /// Test on FPut operation
        /// </summary>
        [TestMethod()]
        [Test]
        public void FPut()
        {
            LogoNumber n1 = new LogoNumber(1000);
            try
            {
                n1.FPut(new LogoWord("2"));
                Assert.IsTrue(false);
            }
            catch (Exception)
            {
                Assert.IsTrue(true);
            }
        }

        /// <summary>
        /// Test on LPut operation
        /// </summary>
        [TestMethod()]
        [Test]
        public void LPut()
        {
            LogoNumber n1 = new LogoNumber(1000);
            try
            {
                n1.LPut(new LogoWord("2"));
                Assert.IsTrue(false);
            }
            catch (Exception)
            {
                Assert.IsTrue(true);
            }
        }

        /// <summary>
        /// Test on Length operation
        /// </summary>
        [TestMethod()]
        [Test]
        public void Length()
        {
            // Various value
            LogoNumber n1 = new LogoNumber(0);
            Assert.AreEqual(n1.Length(), 1);
            LogoNumber n2 = new LogoNumber(1234.5678);
            Assert.AreEqual(n2.Length(), 9);
            LogoNumber n3 = new LogoNumber(123456789);
            Assert.AreEqual(n3.Length(), 9);
        }

        /// <summary>
        /// Test on Sum operation
        /// </summary>
        [TestMethod()]
        [Test]
        public void Sum()
        {
            // Two number
            LogoNumber n1 = new LogoNumber(12000.45);
            LogoNumber n2 = new LogoNumber(328.2);
            LogoNumber n3 = new LogoNumber(128);
            Assert.AreEqual(n1.Sum(n2).Show(), "12328.65");
            Assert.AreEqual(n3.Sum(n3.Sum(n3.Sum(n3))).Show(), "512");

            // Change destination
            LogoWord w4 = new LogoWord("100");
            Assert.AreEqual(n3.Sum(w4).Show(), "228");
            LogoWord w5 = new LogoWord("Logo");
            Assert.AreEqual(n3.Sum(w5).Print(), "128Logo");
        }

        /// <summary>
        /// Test on Difference operation
        /// </summary>
        [TestMethod()]
        [Test]
        public void Difference()
        {
            // Two number
            LogoNumber n1 = new LogoNumber(12510.45);
            LogoNumber n2 = new LogoNumber(305.2);
            LogoNumber n3 = new LogoNumber(128);
            Assert.AreEqual(n1.Difference(n2).Show(), "12205.25");
            Assert.AreEqual(n3.Difference(n3).Show(), "0");

            // Change destination
            LogoWord w4 = new LogoWord("100");
            Assert.AreEqual(n3.Difference(w4).Show(), "28");

            // Exception
            LogoWord w5 = new LogoWord("Logo");
            try
            {
                n3.Difference(w5);
                Assert.IsTrue(false);
            }
            catch (Exception)
            {
                Assert.IsTrue(true);
            }
        }

        /// <summary>
        /// Test on Product operation
        /// </summary>
        [TestMethod()]
        [Test]
        public void Product()
        {
            // Two number
            LogoNumber n1 = new LogoNumber(1200);
            LogoNumber n2 = new LogoNumber(5.5);
            LogoNumber n3 = new LogoNumber(2);
            Assert.AreEqual(n1.Product(n2).Show(), "6600");
            Assert.AreEqual(n3.Product(n3.Product(n3.Product(n3))).Show(), "16");

            // Change destination
            LogoWord w4 = new LogoWord("100.123");
            Assert.AreEqual(n3.Product(w4).Show(), "200.246");

            // Exception
            LogoWord w5 = new LogoWord("Logo");
            try
            {
                n3.Product(w5);
                Assert.IsTrue(false);
            }
            catch (Exception)
            {
                Assert.IsTrue(true);
            }
        }

        /// <summary>
        /// Test on Quotient operation
        /// </summary>
        [TestMethod()]
        [Test]
        public void Quotient()
        {
            // Two number
            LogoNumber n1 = new LogoNumber(1000);
            LogoNumber n2 = new LogoNumber(5.6);
            LogoNumber n3 = new LogoNumber(2);
            Assert.AreEqual(n1.Quotient(n2).Show(), "178.571428571429");
            Assert.AreEqual(n3.Quotient(n3).Show(), "1");

            // Change destination
            LogoWord w4 = new LogoWord("3");
            Assert.AreEqual(n1.Quotient(w4).Show(), "333.333333333333");

            // Exception
            LogoWord w5 = new LogoWord("Logo");
            try
            {
                n3.Quotient(w5);
                Assert.IsTrue(false);
            }
            catch (Exception)
            {
                Assert.IsTrue(true);
            }
        }

        /// <summary>
        /// Test on Minus operation
        /// </summary>
        [TestMethod()]
        [Test]
        public void Minus()
        {
            // Try with few value
            LogoNumber n1 = new LogoNumber(3.14159);
            Assert.AreEqual(n1.Minus().Show(), "-3.14159");
            LogoNumber n2 = new LogoNumber(-3.14159);
            Assert.AreEqual(n2.Minus().Show(), "3.14159");
            Assert.AreEqual(n1.Minus().Minus().Show(), n1.Show());
        }

        /// <summary>
        /// Test on IsEqual operation
        /// </summary>
        [TestMethod()]
        [Test]
        public void IsEqual()
        {
            // Two number
            LogoNumber n1 = new LogoNumber(12000.45);
            LogoNumber n2 = new LogoNumber(12000.45);
            LogoNumber n3 = new LogoNumber(128);
            Assert.IsTrue(n1.IsEqual(n2));
            Assert.IsFalse(n1.IsEqual(n3));

            // Change destination
            LogoWord w4 = new LogoWord("128");
            Assert.IsTrue(n3.IsEqual(w4));
            LogoWord w5 = new LogoWord("Logo");
            Assert.IsFalse(n3.IsEqual(w5));
            LogoNumber n6 = new LogoNumber(3);
            LogoNumber n7 = new LogoNumber(3.0);
            LogoWord w8 = new LogoWord("3");
            Assert.IsTrue(n6.IsEqual(n7));
            Assert.IsTrue(n7.IsEqual(w8));
        }

        /// <summary>
        /// Test on NotEqual operation
        /// </summary>
        [TestMethod()]
        [Test]
        public void NotEqual()
        {
            // Two number
            LogoNumber n1 = new LogoNumber(12000.45);
            LogoNumber n2 = new LogoNumber(12000.45);
            LogoNumber n3 = new LogoNumber(128);
            Assert.IsFalse(n1.NotEqual(n2));
            Assert.IsTrue(n1.NotEqual(n3));

            // Change destination
            LogoWord w4 = new LogoWord("128");
            Assert.IsFalse(n3.NotEqual(w4));
            LogoWord w5 = new LogoWord("Logo");
            Assert.IsTrue(n3.NotEqual(w5));
        }

        /// <summary>
        /// Test on IsGreater operation
        /// </summmary>
        [TestMethod()]
        [Test]
        public void IsGreater()
        {
            // Two number
            LogoNumber n1 = new LogoNumber(22000.45);
            LogoNumber n2 = new LogoNumber(128);
            Assert.IsTrue(n1.IsGreater(n2));
            Assert.IsFalse(n2.IsGreater(n1));

            // Change destination
            LogoWord w3 = new LogoWord("128");
            Assert.IsTrue(n1.IsGreater(w3));
            LogoWord w4 = new LogoWord("Logo");
            Assert.AreEqual(n2.IsGreater(w4), "128".CompareTo("Logo") > 0);
        }

        /// <summary>
        /// Test on IsLower operation
        /// </summary>
        [TestMethod()]
        [Test]
        public void IsLower()
        {
            // Two number
            LogoNumber n1 = new LogoNumber(22000.45);
            LogoNumber n2 = new LogoNumber(128);
            Assert.IsFalse(n1.IsLower(n2));
            Assert.IsTrue(n2.IsLower(n1));

            // Change destination
            LogoWord w3 = new LogoWord("128");
            Assert.IsFalse(n1.IsLower(w3));
            LogoWord w4 = new LogoWord("Logo");
            Assert.AreEqual(n2.IsLower(w4), "128".CompareTo("Logo") < 0);
        }

        /// <summary>
        /// Test on IsGreaterOrEqual operation
        /// </summmary>
        [TestMethod()]
        [Test]
        public void IsGreaterOrEqual()
        {
            // Two number
            LogoNumber n1 = new LogoNumber(1234567.89);
            LogoNumber n2 = new LogoNumber(301269);
            Assert.IsTrue(n1.IsGreaterOrEqual(n2));
            Assert.IsFalse(n2.IsGreaterOrEqual(n1));
            Assert.IsTrue(n2.IsGreaterOrEqual(n2));

            // Change destination
            LogoWord w3 = new LogoWord("301269");
            Assert.IsTrue(n1.IsGreaterOrEqual(w3));
            Assert.IsTrue(n2.IsGreaterOrEqual(w3));
            LogoWord w4 = new LogoWord("Logo");
            Assert.AreEqual(n2.IsGreaterOrEqual(w4), "301269".CompareTo("Logo") >= 0);
        }

        /// <summary>
        /// Test on IsLowerOrEqual operation
        /// </summmary>
        [TestMethod()]
        [Test]
        public void IsLowerOrEqual()
        {
            // Two number
            LogoNumber n1 = new LogoNumber(1234567.89);
            LogoNumber n2 = new LogoNumber(301269);
            Assert.IsFalse(n1.IsLowerOrEqual(n2));
            Assert.IsTrue(n2.IsLowerOrEqual(n1));
            Assert.IsTrue(n2.IsLowerOrEqual(n2));

            // Change destination
            LogoWord w3 = new LogoWord("301269");
            Assert.IsFalse(n1.IsLowerOrEqual(w3));
            Assert.IsTrue(n2.IsLowerOrEqual(w3));
            LogoWord w4 = new LogoWord("Logo");
            Assert.AreEqual(n2.IsLowerOrEqual(w4), "301269".CompareTo("Logo") <= 0);
        }

        /// <summary>
        /// Test on Copy operation
        /// </summmary>
        [TestMethod()]
        [Test]
        public void Copy()
        {
            // Simple value
            LogoNumber n1 = new LogoNumber(3.141519);
            ILogoValue n1c = n1.Copy();
            Assert.IsTrue(n1c.IsNumber());
            Assert.IsTrue(n1c.IsEqual(n1));
            Assert.IsTrue(n1.Print().Equals(n1c.Print()));
            Assert.IsTrue(n1.Show().Equals(n1c.Show()));
        }

        /// <summary>
        /// Test on Show operation
        /// </summmary>
        [TestMethod()]
        [Test]
        public void Show()
        {
            // Integer
            LogoNumber n1 = new LogoNumber(1024);
            Assert.IsTrue(n1.Show().Equals("1024"));

            // Floating point
            LogoNumber n2 = new LogoNumber(379.452);
            Assert.IsTrue(n2.Show().Equals("379.452"));
        }

        /// <summary>
        /// Test on Print operation
        /// </summmary>
        [TestMethod()]
        [Test]
        public void Print()
        {
            // Integer
            LogoNumber n1 = new LogoNumber(1024);
            Assert.IsTrue(n1.Print().Equals("1024"));

            // Floating point
            LogoNumber n2 = new LogoNumber(379.452);
            Assert.IsTrue(n2.Print().Equals((379.452).ToString()));
        }

        /// <summary>
        /// Test on ToString operation
        /// </summmary>
        [TestMethod()]
        [Test]
        public new void ToString()
        {
            // Integer
            LogoNumber n1 = new LogoNumber(1024);
            Assert.IsTrue(n1.ToString().Equals("1024"));

            // Floating point
            LogoNumber n2 = new LogoNumber(379.452);
            Assert.IsTrue(n2.ToString().Equals("379.452"));
        }

        /// <summary>
        /// Test on boolean constant
        /// </summmary>
        [TestMethod()]
        [Test]
        public void TrueFalse()
        {
            LogoNumber n1 = new LogoNumber(0);
            Assert.IsFalse(n1.IsTrue());
            Assert.IsFalse(n1.IsFalse());
            LogoNumber n2 = new LogoNumber(1);
            Assert.IsFalse(n2.IsTrue());
            Assert.IsFalse(n2.IsFalse());
        }

        /// <summary>
        /// Test ToInt and ToDouble
        /// </summary>
        [TestMethod()]
        [Test]
        public void ToIntDouble()
        {
            LogoNumber n1 = new LogoNumber(0);
            Assert.AreEqual(0, n1.ToInt());
            Assert.AreEqual(0.0, n1.ToDouble());
            LogoNumber n2 = new LogoNumber(3.14159);
            Assert.AreEqual(3, n2.ToInt());
            Assert.AreEqual(3.14159, n2.ToDouble());
            LogoNumber n3 = new LogoNumber(-150);
            Assert.AreEqual(-150, n3.ToInt());
            Assert.AreEqual(-150.0, n3.ToDouble());
        }

        /// <summary>
        /// Terminate test.
        /// </summary>
        [TestCleanup()]
        [TearDown]
        public void Terminate()
        {
        }
    }
}

