#region LIOGO - GPL Copyright (c) 2005-2007 Lionel Lask
//
// This file is part of LIOGO.
//
// LIOGO is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// LIOGO is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Liogo; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//================================================================================
#endregion

using System;
#if !TEST_VSNET
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
#endif
using LIOGO.Core;

namespace LIOGO.Test
{
	/// <summary>
	/// Test class for LogoWord.
	/// </summary>
	[TestClass()] [TestFixture]
	public class LogoWordTest
	{
		/// <summary>
		/// Constructor
		/// </summary>
		public LogoWordTest()
		{
		}

		/// <summary>
		/// Init test
		/// </summary>
		[TestInitialize()] [SetUp] public void Init() 
		{
		}

		/// <summary>
		/// Test on constructors
		/// </summary>
        [Test] [TestMethod()] public void Constructor() 
		{
			// Empty value
			LogoWord w1 = new LogoWord();
			Assert.IsNotNull(w1);
			Assert.AreEqual(w1.Show(), "");
			Assert.AreEqual(w1.Print(), "");
			Assert.IsTrue(w1.IsEmpty());
			Assert.IsTrue(w1.IsWord());
			Assert.IsFalse(w1.IsNumber());
			Assert.IsFalse(w1.IsList());
            Assert.IsFalse(w1.IsArray());

			// Single string
			LogoWord w2 = new LogoWord("Hello");
			Assert.IsNotNull(w2);
			Assert.AreEqual(w2.Show(), "Hello");
			Assert.AreEqual(w2.Print(), "Hello");
			Assert.IsFalse(w2.IsEmpty());
			Assert.IsTrue(w2.IsWord());
			Assert.IsFalse(w2.IsNumber());
			Assert.IsFalse(w2.IsList());
            Assert.IsFalse(w2.IsArray());

			// Logo Value
			LogoWord w3 = new LogoWord(new LogoNumber(10));
			Assert.IsNotNull(w3);
			Assert.AreEqual(w3.Show(), "10");
			Assert.AreEqual(w3.Print(), "10");
			Assert.IsFalse(w3.IsEmpty());
			Assert.IsTrue(w3.IsWord());
			Assert.IsFalse(w3.IsNumber());
			Assert.IsFalse(w3.IsList());
            Assert.IsFalse(w3.IsArray());

			LogoWord w5 = new LogoWord(new LogoWord("Hello"));
			Assert.IsNotNull(w5);
			Assert.AreEqual(w5.Show(), "Hello");
			Assert.AreEqual(w5.Print(), "Hello");
			Assert.IsFalse(w5.IsEmpty());
			Assert.IsTrue(w5.IsWord());
			Assert.IsFalse(w5.IsNumber());
			Assert.IsFalse(w5.IsList());
            Assert.IsFalse(w5.IsArray());
	
			// Boolean value
			LogoWord w6 = new LogoWord(true);
			Assert.IsTrue(w6.IsTrue());
			Assert.AreEqual("true", w6.Print());
			LogoWord w7 = new LogoWord(false);
			Assert.IsTrue(w7.IsFalse());
			Assert.AreEqual("false", w7.Print());
		}

		/// <summary>
		/// Test on Item operation
		/// </summary>
		[TestMethod()] [Test] public void Item() 
		{
			// Empty string
			LogoWord w1 = new LogoWord();
			try
			{
				w1.Item(new LogoNumber(-1));
				Assert.IsTrue(false);
			} 
			catch (Exception)
			{
				Assert.IsTrue(true);
			}
			try
			{
				w1.Item(new LogoNumber(0));
				Assert.IsTrue(false);
			} 
			catch (Exception)
			{
				Assert.IsTrue(true);
			}
			try
			{
				w1.Item(new LogoNumber(1));
				Assert.IsTrue(false);
			} 
			catch (Exception)
			{
				Assert.IsTrue(true);
			}
			try
			{
				w1.Item(new LogoWord("foo"));
				Assert.IsTrue(false);
			} 
			catch (Exception)
			{
				Assert.IsTrue(true);
			}

			// Single string
			LogoWord w2 = new LogoWord("Hello");
			try
			{
				w2.Item(new LogoNumber(-1));
				Assert.IsTrue(false);
			} 
			catch (Exception)
			{
				Assert.IsTrue(true);
			}
			try
			{
				w2.Item(new LogoNumber(0));
				Assert.IsTrue(false);
			} 
			catch (Exception)
			{
				Assert.IsTrue(true);
			}
			Assert.IsTrue(w2.Item(new LogoNumber(1)).IsEqual(new LogoWord("H")));
			Assert.IsTrue(w2.Item(new LogoNumber(1.5F)).IsEqual(new LogoWord("H")));
			Assert.IsTrue(w2.Item(new LogoNumber(5)).IsEqual(new LogoWord("o")));
			try
			{
				w2.Item(new LogoNumber(10));
				Assert.IsTrue(false);
			} 
			catch (Exception)
			{
				Assert.IsTrue(true);
			}
			try
			{
				w2.Item(new LogoNumber(1234.5678));
				Assert.IsTrue(false);
			} 
			catch (Exception)
			{
				Assert.IsTrue(true);
			}

			Assert.IsTrue(w2.Item(new LogoWord("1")).IsEqual(new LogoWord("H")));
			Assert.IsTrue(w2.Item(new LogoWord("1.5")).IsEqual(new LogoWord("H")));
		}

		/// <summary>
		/// Test on First operation
		/// </summary>
		[TestMethod()] [Test] public void First() 
		{
			// Empty string
			LogoWord w1 = new LogoWord();
			Assert.IsTrue(w1.First().IsEmpty());

			// Single string
			LogoWord w2 = new LogoWord("Logo");
			Assert.IsTrue(w2.First().IsEqual(new LogoWord("L")));
			Assert.IsTrue(w2.First().Print().Equals("L"));
			Assert.IsTrue(w2.First().First().First().Print().Equals("L"));
		}

		/// <summary>
		/// Test on Last operation
		/// </summary>
		[TestMethod()] [Test] public void Last() 
		{
			// Empty string
			LogoWord w1 = new LogoWord();
			Assert.IsTrue(w1.Last().IsEmpty());

			// Single string
			LogoWord w2 = new LogoWord("Hello");
			Assert.IsTrue(w2.Last().IsEqual(new LogoWord("o")));
			Assert.IsTrue(w2.Last().Print().Equals("o"));
			Assert.IsTrue(w2.Last().Last().Last().Print().Equals("o"));
		}

		/// <summary>
		/// Test on ButFirst operation
		/// </summary>
		[TestMethod()] [Test] public void ButFirst() 
		{
			// Empty string
			LogoWord w1 = new LogoWord();
			Assert.IsTrue(w1.ButFirst().IsEmpty());
			Assert.IsTrue(w1.ButFirst().ButFirst().ButFirst().IsEmpty());

			// Single string
			LogoWord w2 = new LogoWord("Word");
			Assert.IsTrue(w2.ButFirst().IsEqual(new LogoWord("ord")));
			Assert.IsTrue(w2.ButFirst().Print().Equals("ord"));
			Assert.IsTrue(w2.ButFirst().ButFirst().ButFirst().Print().Equals("d"));
			Assert.IsTrue(w2.ButFirst().ButFirst().ButFirst().ButFirst().IsEmpty());
		}

		/// <summary>
		/// Test on ButLast operation
		/// </summary>
		[TestMethod()] [Test] public void ButLast() 
		{
			// Empty string
			LogoWord w1 = new LogoWord();
			Assert.IsTrue(w1.ButLast().IsEmpty());
			Assert.IsTrue(w1.ButLast().ButLast().ButLast().IsEmpty());

			// Single string
			LogoWord w2 = new LogoWord("Word");
			Assert.IsTrue(w2.ButLast().IsEqual(new LogoWord("Wor")));
			Assert.IsTrue(w2.ButLast().Print().Equals("Wor"));
			Assert.IsTrue(w2.ButLast().ButLast().ButLast().Print().Equals("W"));
			Assert.IsTrue(w2.ButLast().ButLast().ButLast().ButLast().IsEmpty());
		}

		/// <summary>
		/// Test on FPut operation
		/// </summary>
		[TestMethod()] [Test] public void FPut() 
		{
			LogoWord w1 = new LogoWord("foo");
			try
			{
				w1.FPut(new LogoWord("R"));
				Assert.IsTrue(false);
			} 
			catch (Exception)
			{
				Assert.IsTrue(true);
			}
		}

		/// <summary>
		/// Test on LPut operation
		/// </summary>
		[TestMethod()] [Test] public void LPut() 
		{
			LogoWord w1 = new LogoWord("foo");
			try
			{
				w1.LPut(new LogoWord("R"));
				Assert.IsTrue(false);
			} 
			catch (Exception)
			{
				Assert.IsTrue(true);
			}
		}

		/// <summary>
		/// Test on Length operation
		/// </summary>
		[TestMethod()] [Test] public void Length() 
		{
			// Empty string
			LogoWord w1 = new LogoWord();
			Assert.AreEqual(w1.Length(),0);

			// Single string
			LogoWord w2 = new LogoWord("Logo");
			Assert.AreEqual(w2.Length(),4);
			LogoWord w3 = new LogoWord("L");
			Assert.AreEqual(w3.Length(),1);
		}

		/// <summary>
		/// Test on Sum operation
		/// </summary>
		[TestMethod()] [Test] public void Sum() 
		{
			// Empty string (one or both)
			LogoWord w1 = new LogoWord("");
			LogoWord w2 = new LogoWord("");
			LogoWord w3 = new LogoWord("Hello");
			Assert.IsTrue(w1.Sum(w2).IsEmpty());
			Assert.AreEqual(w1.Sum(w3).Print(), "Hello");
			Assert.AreEqual(w3.Sum(w1).Print(), "Hello");

			// Two string
			LogoWord w4 = new LogoWord("Lo");
			LogoWord w5 = new LogoWord("go");
			Assert.AreEqual(w4.Sum(w5).Print(), "Logo");
			Assert.AreEqual(w3.Sum(w4.Sum(w5.Sum(w1))).Print(), "HelloLogo");

			// Change destination
			LogoWord w6 = new LogoWord("100");
			LogoWord w7 = new LogoWord("23");
			Assert.AreEqual(w6.Sum(w7).Print(), "123");

            LogoWord w8 = new LogoWord("C");
            Assert.AreEqual(w8.Sum(new LogoWord()).Print(), "C");

			// Mixed destination
			Assert.AreEqual(w4.Sum(new LogoNumber(45.5)).Show(), "Lo45.5");
		
		}

		/// <summary>
		/// Test on Difference operation
		/// </summary>
		[TestMethod()] [Test] public void Difference() 
		{
			// Try with two word (autoconvert to number)
			LogoWord w1 = new LogoWord("150.23");
			LogoWord w2 = new LogoWord("10.10");
			Assert.AreEqual(w1.Difference(w2).Show(), "140.13");

			// Try with one word (autoconvert to number) and one number
			LogoNumber n3 = new LogoNumber(10.10);
			Assert.AreEqual(w1.Difference(n3).Show(), "140.13");

			// Try with a list
            try
            {
                LogoWord w4 = new LogoWord("34");
                w4.Difference(new LogoWord("A")); 
                Assert.IsTrue(false);
            }
            catch (Exception)
            {
            }
        }

		/// <summary>
		/// Test on Product operation
		/// </summary>
		[TestMethod()] [Test] public void Product() 
		{
			// Try with two word (autoconvert to number)
			LogoWord w1 = new LogoWord("150");
			LogoWord w2 = new LogoWord("2");
			Assert.AreEqual(w1.Product(w2).Show(), "300");

			// Try with one word (autoconvert to number) and one number
			LogoNumber n3 = new LogoNumber(2);
			Assert.AreEqual(w1.Product(n3).Show(), "300");

			// Try with a list
            try
            {
                LogoWord w5 = new LogoWord("34");
                w5.Product(new LogoWord("A"));
                Assert.IsTrue(false);
            }
            catch (Exception)
            {
                Assert.IsTrue(true);
            }
            LogoWord w6 = new LogoWord("A");
            try
            {
                w6.Product(new LogoNumber(123));
                Assert.IsTrue(false);
            }
            catch (Exception)
            {
                Assert.IsTrue(true);
            }
		}

		/// <summary>
		/// Test on Quotient operation
		/// </summary>
		[TestMethod()] [Test] public void Quotient() 
		{
			// Try with two word (autoconvert to number)
			LogoWord w1 = new LogoWord("150");
			LogoWord w2 = new LogoWord("2");
			Assert.AreEqual(w1.Quotient(w2).Show(), "75");

			// Try with one word (autoconvert to number) and one number
			LogoNumber n3 = new LogoNumber(2);
			Assert.AreEqual(w1.Quotient(n3).Show(), "75");

			// Try with a list
            try
            {
                LogoWord w5 = new LogoWord("34");
                w5.Quotient(new LogoWord("A"));
                Assert.IsTrue(false);
            }
            catch (Exception)
            {
                Assert.IsTrue(true);
            }
		}

		/// <summary>
		/// Test on Minus operation
		/// </summary>
		[TestMethod()] [Test] public void Minus() 
		{
			// Try with one word (autoconvert to number)
			LogoWord w1 = new LogoWord("3.14159");
			Assert.AreEqual(w1.Minus().Show(), "-3.14159");
			LogoWord w2 = new LogoWord("-3.14159");
			Assert.AreEqual(w2.Minus().Show(), "3.14159");
			Assert.AreEqual(w1.Minus().Minus().Show(), w1.Print());

			// Try with a string
			LogoWord w3 = new LogoWord("Logo");
			try
			{
				w3.Minus();
				Assert.IsTrue(false);
			}
			catch (Exception)
			{
				Assert.IsTrue(true);
			}
		}

		/// <summary>
		/// Test on IsEqual operation
		/// </summary>
		[TestMethod()] [Test] public void IsEqual() 
		{
			// Empty string (one or both)
			LogoWord w1 = new LogoWord("");
			LogoWord w2 = new LogoWord("");
			LogoWord w3 = new LogoWord("Hello");
			Assert.IsTrue(w1.IsEqual(w2));
			Assert.IsFalse(w1.IsEqual(w3));
			Assert.IsFalse(w3.IsEqual(w1));

			// Two string
			LogoWord w4 = new LogoWord("Lo");
			LogoWord w5 = new LogoWord("go");
			LogoWord w6 = new LogoWord("Lo");
			Assert.IsFalse(w4.IsEqual(w5));
			Assert.IsTrue(w4.IsEqual(w6));
			Assert.IsTrue(w4.Sum(w5).IsEqual(w6.Sum(w5)));
			Assert.IsTrue(w4.Sum(w5).First().IsEqual(w6.First()));

			// Mixed destination
			LogoWord w7 = new LogoWord("100");
			LogoNumber n8 = new LogoNumber(100);
			LogoNumber n9 = new LogoNumber(10);
			Assert.IsTrue(w7.IsEqual(n8));
			Assert.IsFalse(w7.IsEqual(n9));
			Assert.IsFalse(w4.IsEqual(n9));
		}

		/// <summary>
		/// Test on NotEqual operation
		/// </summary>
		[TestMethod()] [Test] public void NotEqual() 
		{
			// Empty string (one or both)
			LogoWord w1 = new LogoWord("");
			LogoWord w2 = new LogoWord("");
			LogoWord w3 = new LogoWord("Hello");
			Assert.IsFalse(w1.NotEqual(w2));
			Assert.IsTrue(w1.NotEqual(w3));
			Assert.IsTrue(w3.NotEqual(w1));

			// Two string
			LogoWord w4 = new LogoWord("Lo");
			LogoWord w5 = new LogoWord("go");
			LogoWord w6 = new LogoWord("Lo");
			Assert.IsTrue(w4.NotEqual(w5));
			Assert.IsFalse(w4.NotEqual(w6));
			Assert.IsFalse(w4.Sum(w5).NotEqual(w6.Sum(w5)));
			Assert.IsFalse(w4.Sum(w5).First().NotEqual(w6.First()));

			// Mixed destination
			LogoWord w7 = new LogoWord("100");
			LogoNumber n8 = new LogoNumber(100);
			LogoNumber n9 = new LogoNumber(10);
			Assert.IsFalse(w7.NotEqual(n8));
			Assert.IsTrue(w7.NotEqual(n9));
			Assert.IsTrue(w4.NotEqual(n9));
            Assert.IsFalse(w7.NotEqual(w7));
		}

		/// <summary>
		/// Test on IsGreater operation
		/// </summmary>
		[TestMethod()] [Test] public void IsGreater() 
		{
			// Empty string (one or both)
			LogoWord w1 = new LogoWord("");
			LogoWord w2 = new LogoWord("");
			LogoWord w3 = new LogoWord("Hello");
			Assert.AreEqual(w1.IsGreater(w2), "".CompareTo("")>0);
			Assert.AreEqual(w2.IsGreater(w3), "".CompareTo("Hello")>0);
			Assert.AreEqual(w3.IsGreater(w1), "Hello".CompareTo("")>0);

			// Two string
			LogoWord w4 = new LogoWord("Lo");
			LogoWord w5 = new LogoWord("Go");
			Assert.AreEqual(w4.IsGreater(w5), "Lo".CompareTo("Go")>0);
			Assert.AreEqual(w5.IsGreater(w4), "Go".CompareTo("Lo")>0);

			// Mixed destination
			LogoWord w7 = new LogoWord("300");
			LogoNumber n8 = new LogoNumber(10);
			Assert.IsTrue(w7.IsGreater(n8));
			LogoWord w9 = new LogoWord("3");
			LogoNumber n10 = new LogoNumber(10);
			Assert.IsFalse(w9.IsGreater(n10));
            Assert.IsTrue(w7.IsGreater(new LogoWord("200")));
		}

		/// <summary>
		/// Test on IsLower operation
		/// </summary>
		[TestMethod()] [Test] public void IsLower() 
		{
			// Empty string (one or both)
			LogoWord w1 = new LogoWord("");
			LogoWord w2 = new LogoWord("");
			LogoWord w3 = new LogoWord("Hello");
			Assert.AreEqual(w1.IsLower(w2), "".CompareTo("")<0);
			Assert.AreEqual(w2.IsLower(w3), "".CompareTo("Hello")<0);
			Assert.AreEqual(w3.IsLower(w1), "Hello".CompareTo("")<0);

			// Two string
			LogoWord w4 = new LogoWord("Lo");
			LogoWord w5 = new LogoWord("Go");
			Assert.AreEqual(w4.IsLower(w5), "Lo".CompareTo("Go")<0);
			Assert.AreEqual(w5.IsLower(w4), "Go".CompareTo("Lo")<0);

			// Mixed destination
			LogoWord w7 = new LogoWord("300");
			LogoNumber n8 = new LogoNumber(10);
			Assert.IsFalse(w7.IsLower(n8));
			LogoWord w9 = new LogoWord("3");
			LogoNumber n10 = new LogoNumber(10);
			Assert.IsTrue(w9.IsLower(n10));
            Assert.IsFalse(w7.IsLower(new LogoWord("200")));
		}

		/// <summary>
		/// Test on IsGreaterOrEqual operation
		/// </summmary>
		[TestMethod()] [Test] public void IsGreaterOrEqual() 
		{
			// Empty string (one or both)
			LogoWord w1 = new LogoWord("");
			LogoWord w2 = new LogoWord("");
			LogoWord w3 = new LogoWord("Hello");
			Assert.IsTrue(w1.IsGreaterOrEqual(w2));
			Assert.AreEqual(w2.IsGreaterOrEqual(w3), "".CompareTo("Hello")>=0);
			Assert.AreEqual(w3.IsGreaterOrEqual(w1), "Hello".CompareTo("")>=0);

			// Two string
			LogoWord w4 = new LogoWord("Lo");
			LogoWord w5 = new LogoWord("Go");
			LogoWord w6 = new LogoWord("Lo");
			Assert.IsTrue(w4.IsGreaterOrEqual(w6));
			Assert.AreEqual(w4.IsGreaterOrEqual(w5), "Lo".CompareTo("Go")>=0);
			Assert.AreEqual(w5.IsGreaterOrEqual(w4), "Go".CompareTo("Lo")>=0);

			// Mixed destination
			LogoWord w7 = new LogoWord("300");
			LogoNumber n8 = new LogoNumber(10);
			Assert.IsTrue(w7.IsGreaterOrEqual(n8));
			LogoWord w9 = new LogoWord("3");
			LogoNumber n10 = new LogoNumber(10);
			Assert.IsFalse(w9.IsGreaterOrEqual(n10));
			LogoNumber n11 = new LogoNumber(300);
			Assert.IsTrue(w7.IsGreaterOrEqual(n11));
            Assert.IsTrue(w7.IsGreaterOrEqual(new LogoWord("300")));
		}

		/// <summary>
		/// Test on IsLowerOrEqual operation
		/// </summmary>
		[TestMethod()] [Test] public void IsLowerOrEqual() 
		{
			// Empty string (one or both)
			LogoWord w1 = new LogoWord("");
			LogoWord w2 = new LogoWord("");
			LogoWord w3 = new LogoWord("Hello");
			Assert.IsTrue(w1.IsLowerOrEqual(w2));
			Assert.AreEqual(w2.IsLowerOrEqual(w3), "".CompareTo("Hello")<=0);
			Assert.AreEqual(w3.IsLowerOrEqual(w1), "Hello".CompareTo("")<=0);

			// Two string
			LogoWord w4 = new LogoWord("Lo");
			LogoWord w5 = new LogoWord("Go");
			LogoWord w6 = new LogoWord("Lo");
			Assert.IsTrue(w4.IsLowerOrEqual(w6));
			Assert.AreEqual(w4.IsLowerOrEqual(w5), "Lo".CompareTo("Go")<=0);
			Assert.AreEqual(w5.IsLowerOrEqual(w4), "Go".CompareTo("Lo")<=0);

			// Mixed destination
			LogoWord w7 = new LogoWord("300");
			LogoNumber n8 = new LogoNumber(10);
			Assert.IsFalse(w7.IsLowerOrEqual(n8));
			LogoWord w9 = new LogoWord("3");
			LogoNumber n10 = new LogoNumber(10);
			Assert.IsTrue(w9.IsLowerOrEqual(n10));
			LogoNumber n11 = new LogoNumber(300);
			Assert.IsTrue(w7.IsLowerOrEqual(n11));
            Assert.IsFalse(w7.IsLowerOrEqual(new LogoWord("200")));
		}

		/// <summary>
		/// Test on Copy operation
		/// </summmary>
		[TestMethod()] [Test] public void Copy() 
		{
			// Empty string
			LogoWord w1 = new LogoWord("");
			ILogoValue w1c = w1.Copy();
			Assert.IsTrue(w1c.IsWord());
			Assert.IsTrue(w1c.IsEqual(w1));
			Assert.IsTrue(w1.Print().Equals(w1c.Print()));
			Assert.IsTrue(w1.Show().Equals(w1c.Show()));

			// Simple string
			LogoWord w2 = new LogoWord("Hello");
			ILogoValue w2c = w2.Copy();
			Assert.IsTrue(w1c.IsWord());
			Assert.IsTrue(w2c.IsEqual(w2));
			Assert.IsTrue(w2.Print().Equals(w2c.Print()));
			Assert.IsTrue(w2.Show().Equals(w2c.Show()));
		}

		/// <summary>
		/// Test on Show operation
		/// </summmary>
		[TestMethod()] [Test] public void Show() 
		{
			// Empty string
			LogoWord w1 = new LogoWord("");
			Assert.IsTrue(w1.Show().Equals(""));

			// Simple string
			LogoWord w2 = new LogoWord("Hello");
			Assert.IsTrue(w2.Show().Equals("Hello"));

			// White space in string
			LogoWord w3 = new LogoWord("Hello World!");
			Assert.IsTrue(w3.Show().Equals("Hello World!"));
			LogoWord w4 = new LogoWord("It's\r\na\r\nlong\r\nline");
			Assert.IsTrue(w4.Show().Equals("It's\r\na\r\nlong\r\nline"));

		}

		/// <summary>
		/// Test on Print operation
		/// </summmary>
		[TestMethod()] [Test] public void Print() 
		{
			// Empty string
			LogoWord w1 = new LogoWord("");
			Assert.IsTrue(w1.Print().Equals(""));

			// Simple string
			LogoWord w2 = new LogoWord("Hello");
			Assert.IsTrue(w2.Print().Equals("Hello"));

			// White space in string
			LogoWord w3 = new LogoWord("Hello World!");
			Assert.IsTrue(w3.Print().Equals("Hello World!"));
		}

		/// <summary>
		/// Test on ToString operation
		/// </summmary>
		[TestMethod()] [Test] public new void ToString() 
		{
			// Default quoted
			LogoWord w1 = new LogoWord("");
			Assert.IsTrue(w1.ToString().Equals("\""));
			LogoWord w2 = new LogoWord("Hello");
			Assert.IsTrue(w2.ToString().Equals("\"Hello"));
			LogoWord w3 = new LogoWord("Hello World!");
			Assert.IsTrue(w3.ToString().Equals("\"|Hello World!|"));

			// Quoted
			LogoWord w7 = new LogoWord("", true);
			Assert.IsTrue(w7.ToString().Equals("\""));
			LogoWord w8 = new LogoWord("Hello", true);
			Assert.IsTrue(w8.ToString().Equals("\"Hello"));
			LogoWord w9 = new LogoWord("Hello World!", true);
			Assert.IsTrue(w9.ToString().Equals("\"|Hello World!|"));

			// Non quoted
			LogoWord w4 = new LogoWord("", false);
			Assert.IsTrue(w4.ToString().Equals(""));
			LogoWord w5 = new LogoWord("Hello", false);
			Assert.IsTrue(w5.ToString().Equals("Hello"));
			LogoWord w6 = new LogoWord("Hello World!", false);
			Assert.IsTrue(w6.ToString().Equals("Hello World!"));
		}

		/// <summary>
		/// Test on boolean constant
		/// </summmary>
		[TestMethod()] [Test] public void TrueFalse() 
		{
			// Basic test
			Assert.IsTrue(LogoWord.True.IsTrue());
			Assert.IsFalse(LogoWord.True.IsFalse());
			Assert.IsTrue(LogoWord.False.IsFalse());
			Assert.IsFalse(LogoWord.False.IsTrue());

			// Empty string
			LogoWord w1 = new LogoWord("");
			Assert.IsFalse(w1.IsTrue());
			Assert.IsFalse(w1.IsFalse());

			// Simple string
			LogoWord w2 = new LogoWord("Hello");
			Assert.IsFalse(w2.IsTrue());
			Assert.IsFalse(w2.IsFalse());

			// String true
			LogoWord w3 = new LogoWord("true");
			Assert.IsTrue(w3.IsTrue());
			Assert.IsFalse(w3.IsFalse());
			LogoWord w4 = new LogoWord("True");
			Assert.IsTrue(w4.IsTrue());
			Assert.IsFalse(w4.IsFalse());

			// String false
			LogoWord w5 = new LogoWord("false");
			Assert.IsTrue(w5.IsFalse());
			Assert.IsFalse(w5.IsTrue());
			LogoWord w6 = new LogoWord("False");
			Assert.IsTrue(w6.IsFalse());
			Assert.IsFalse(w6.IsTrue());
		}

		/// <summary>
		/// Test on Quoted operation
		/// </summmary>
		[TestMethod()] [Test] public void Quoted() 
		{
			// Two quoted string give a quoted string
			LogoWord w1 = new LogoWord("He");
			Assert.AreEqual("\"He", w1.ToString());
			LogoWord w2 = new LogoWord("llo");
			Assert.AreEqual("\"llo", w2.ToString());
			Assert.AreEqual(w1.Sum(w2).ToString(), "\"Hello");

			// Two nonquoted string give a quoted string
			LogoWord w3 = new LogoWord("He", false);
			Assert.AreEqual("He", w3.ToString());
			LogoWord w4 = new LogoWord("llo", false);
			Assert.AreEqual("llo", w4.ToString());
			Assert.AreEqual(w3.Sum(w4).ToString(), "\"Hello");

			// One nonquoted string and one quoted string give a quoted string
			LogoWord w5 = new LogoWord("He", false);
			Assert.AreEqual("He", w5.ToString());
			LogoWord w6 = new LogoWord("llo", true);
			Assert.AreEqual("\"llo", w6.ToString());
			Assert.AreEqual(w5.Sum(w6).ToString(), "\"Hello");
		}

		/// <summary>
		/// Terminate test.
		/// </summary>
		[TestCleanup()] [TearDown] public void Terminate() 
		{
		}
	}
}
