﻿using MyJScript;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace MyDLRUnitTest
{
    /// <summary>
    ///This is a test class for MJSStringTest and is intended
    ///to contain all MJSStringTest Unit Tests
    ///</summary>
    [TestClass()]
    public class MJSStringTest
    {
        /// <summary>
        ///A test for MJSString Constructor
        ///</summary>
        [TestMethod()]
        public void MJSStringConstructorTest()
        {
            MJSString s1 = new MJSString(null);
            Assert.IsNull(s1.ValueOf());
            Assert.AreEqual("null", s1.ToString());

            MJSString s2 = new MJSString("MyJScript");
            Assert.AreEqual("MyJScript", s2.ValueOf());
            Assert.AreEqual("\"MyJScript\"", s2.ToString());

            MJSString s3 = new MJSString("");
            Assert.AreEqual("", s3.ValueOf());
            Assert.AreEqual("\"\"", s3.ToString());

            s3.Set("value", new MJSString("200"));
            Assert.IsNull(s3.Get("value"));
            Assert.AreEqual("\"\"", s3.ToString());
        }

        /// <summary>
        ///A test for MJSString Primitives
        ///</summary>
        [TestMethod()]
        public void MJSStringPrimitivesTest()
        {
            MJSString s1 = new MJSString("Hello");
            Assert.AreEqual(typeof(MJSFunction), s1.Get("blink").GetType());
            MJSFunction f1 = s1.Get("blink") as MJSFunction;
            MJSObject o1 = f1.Call(s1);
            Assert.AreEqual(typeof(MJSString), o1.GetType());
            Assert.AreEqual("\"<blink>Hello</blink>\"", o1.ToString());
            MJSObject o4 = s1.Get("length");
            Assert.AreEqual(typeof(MJSNumber), o4.GetType());
            Assert.AreEqual(5, (int)o4.ValueOf());

            MJSString s2 = new MJSString("DNG");
            Assert.AreEqual(typeof(MJSFunction), s2.Get("bold").GetType());
            MJSFunction f2 = s2.Get("bold") as MJSFunction;
            MJSObject o2 = f2.Call(s2);
            Assert.AreEqual(typeof(MJSString), o2.GetType());
            MJSObject o3 = s2.Get("length");
            Assert.AreEqual(typeof(MJSNumber), o3.GetType());
            Assert.AreEqual(3, (int)o3.ValueOf());

            MJSString s3 = new MJSString("DotNetGuru");
            Assert.AreEqual(typeof(MJSFunction), s3.Get("substr").GetType());
            MJSFunction f3 = s3.Get("substr") as MJSFunction;
            MJSObject o5 = f3.Call(s3, new MJSNumber(3), new MJSNumber(4));
            Assert.AreEqual(typeof(MJSString), o5.GetType());
            Assert.AreEqual("NetG", (string)o5.ValueOf());
        }
    }
}
