﻿using System;
using System.ComponentModel;
using System.Globalization;
using MyDLR;


namespace MyJScript
{
    /// <summary>
    /// MyJScript converter.
    /// </summary>
    public class MJSConverter : TypeConverter
    {
        /// <summary>
        /// Does the specified destination could be converted to MJSObject ?
        /// </summary>
        /// <param name="sourceType">source destination</param>
        /// <returns>true if destination can convert to MJSObject</returns>
        public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
        {
            if (sourceType == typeof(string) || sourceType == typeof(Int32))
                return true;

            return false;
        }

        /// <summary>
        /// Convert a destination to a MJSObject.
        /// </summary>
        /// <param name="value">value to convert</param>
        /// <returns>result of conversion</returns>
        public override object ConvertFrom(ITypeDescriptorContext context, CultureInfo culture, object value)
        {
            if (value is string)
                return new MJSString(value as string);
            else if (value is Int32)
                return new MJSNumber((int)value);

            throw new NotSupportedException(value.ToString());
        }

        /// <summary>
        /// Does a ILogoValue could be converted to the specifed destination ?
        /// </summary>
        /// <param name="destinationType">destination destination</param>
        /// <returns>true if could be convert</returns>
        public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
        {
            if (destinationType == typeof(string) ||
                destinationType == typeof(Int32))
                return true;
            else
                return false;
        }

        /// <summary>
        /// Convert to a destination.
        /// </summary>
        /// <param name="value">value to convert</param>
        /// <param name="destinationType">destination destination</param>
        /// <returns>value to the specified destination</returns>
        public override object ConvertTo(ITypeDescriptorContext context, CultureInfo culture, object value, Type destinationType)
        {
            MJSObject mjsvalue = value as MJSObject;
            if (destinationType == typeof(string))
            {
                if (value is MJSString)
                    return mjsvalue.ValueOf();
                else if (value is MJSNumber)
                    return mjsvalue.ToString();
            }
            else if (destinationType == typeof(Int32))
            {
                if (value is MJSNumber)
                    return mjsvalue.ValueOf();
            }

            throw new NotSupportedException(destinationType.ToString());
        }
    }
}

