﻿using System;

namespace MyJScript
{
    /// <summary>
    /// Class for a function.
    /// </summary>
    public class MJSFunction : MJSObject
    {
        /// <summary>
        /// Delegate for function code.
        /// </summary>
        public delegate MJSObject FunctionProcess(MJSObject @this, params MJSObject[] args);

        /// <summary>
        /// Function instructions.
        /// </summary>
        FunctionProcess instructions;

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="instructions">instructions for this function</param>
        public MJSFunction(FunctionProcess instructions)
        {
            this.instructions = instructions;
        }

        /// <summary>
        /// Call the function.
        /// </summary>
        /// <param name="this">this value</param>
        /// <param name="args">argument array</param>
        /// <returns>return value</returns>
        public MJSObject Call(MJSObject @this, params MJSObject[] args)
        {
            if (instructions != null)
                return instructions(@this, args);

            return null;
        }

        /// <summary>
        /// Get the value.
        /// </summary>
        /// <returns>value</returns>
        public override object ValueOf()
        {
            return instructions;
        }

        /// <summary>
        /// Convert to string.
        /// </summary>
        /// <returns>A dummy function declaration</returns>
        public override string ToString()
        {
            // Cant print the source code because it's a delegate !
            return "function() {}";
        }
    }
}
