﻿using System;
using System.ComponentModel;
using MyDLR;
using LIOGO.Core;
using MyJScript;

namespace MyDLRTest
{
    /// <summary>
    /// Test class for MyDLR.
    /// </summary>
    class Program
    {
        #region Init MyJScript Cat class
        static MJSObject CatMiaow(MJSObject @this, params MJSObject[] args)
        {
            Console.WriteLine((@this.Get("name").ValueOf() as string) + " said Miaow");
            return null;
        }

        static MJSObject CatConstructor(MJSObject @this, params MJSObject[] args)
        {
            @this.Set("name", args[0]);
            @this.Set("miaow", new MJSFunction(CatMiaow));
            return null;
        }
        #endregion

        static void Main(string[] args)
        {
            #region Init MyDLR
            // Add providers for each language in DLR
            Providers.Add(new MJSDLRProvider());
            Providers.Add(new LogoDLRProvider());

            // Create MyJScript Cat class
            MJSContext.GetContext().Set("Cat", new MJSFunction(CatConstructor));
            #endregion

            // Create a MJSString object
            object mjs = "String".CreateInstance("DotNetGuru");
            PrintValue(mjs);
            PrintValue(mjs.GetProperty("length"));
            PrintValue(mjs.CallMethod("substr", 6));

            // Create an MyJScript Cat object
            object mjc = "Cat".CreateInstance("Felix");
            PrintValue(mjc);
            PrintValue(mjc.GetProperty("name"));
            mjc.CallMethod("miaow");

            // Create a System.String object
            object s = "System.String".CreateInstance(new Char[3] {'A', 'B', 'C'});
            PrintValue(s.GetProperty("length"));
            PrintValue(s.CallMethod("indexof", "B"));

            // Create a Liogo Word object
            object lw = "Word".CreateInstance("Hello World !");
            PrintValue(lw);
            PrintValue(lw.GetProperty("count"));
            PrintValue(lw.CallMethod("butfirst"));

            // Interop between DLR types
            object c = "Word".CreateInstance("B");
            PrintValue(s.CallMethod("indexof", c));
            object t = "String".CreateInstance("2");
            PrintValue(lw.CallMethod("item", t));

            Console.WriteLine("[Press ENTER to quit]");
            Console.ReadLine();
        }

        /// <summary>
        /// Print destination value as string.
        /// </summary>
        /// <param name="o">object to print</param>
        static void PrintValue(Object o)
        {
            Console.WriteLine(String.Format("[{0}]\t[{1}]\t\t{2}", (o is ICallable)?"DLR":"CLR", o.GetType().ToString(), o.ToString()));
        }
    }
}
